#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>  // open()
#include <sys/stat.h>   // open()
#include <fcntl.h>      // open()
#include <linux/i2c-dev.h>
#include "CSensorColor.h"

#define SENSOR_ADDR 0x10



CSensorColor::CSensorColor(unsigned char ucAddr)
{
    /* 局所変数宣言 */
    char szDevice[64];    // デバイス名
    int iRet;  // 戻り値


    /* デバイスオープン */
    sprintf(szDevice, "/dev/i2c-1");
    m_fd = open(szDevice, O_RDWR);
    if (0 > m_fd) {
        fprintf(stderr, "device open error!\n");
        //exit(-1);
    }
    usleep(200 * 1000);

    /* I2Cスレーブの設定 */
    iRet = ioctl(m_fd, I2C_SLAVE, SENSOR_ADDR);
    if (0 > iRet) {
        fprintf(stderr, "ioctl error!\n");
        close(m_fd);
    }
    usleep(200 * 1000);

    return;
}

CSensorColor::~CSensorColor()
{
    /* デバイスクローズ */
    if (0 <= m_fd) {
        close(m_fd);
    }

    return;
}

int CSensorColor::initDevice(void)
{
    /* 局所変数宣言 */
    int iRet;  // 戻り値
    unsigned short uiData;

    /* *** CONF Register の設定 *** */
    // X[7]: 0: Reserved
    // IT[6-4]: 001: Integral Time = 80ms
    // X[3]: 0: Reserved
    // TRIG[2]: 0: No Trigger
    // AF[1]: 0: Auto Mode
    // SD[0]: 0: Device Enable

    /* 一旦、Disableにする */
    iRet = siWriteRegWord(0x00, 0x11);
    if (RET_SUCCESS != iRet) {
        return iRet;
    }

    /* Enableにする */
    iRet = siWriteRegWord(0x00, 0x10);
    if (RET_SUCCESS != iRet) {
        return iRet;
    }

    return RET_SUCCESS;
}

int CSensorColor::siWriteRegWord(
    unsigned char ucAddr,
    unsigned short uiWriteData)
{
    /* 局所変数宣言 */
    int iRet;  // 戻り値
    unsigned short uiReadData;  // 読み出し値

    if (0 > m_fd) {
        return RET_INVALID_FD;
    }

    iRet = i2c_smbus_write_word_data(m_fd, ucAddr, uiWriteData);
    usleep(200 * 1000);
    /* レジスタへの書き込みが失敗している場合 */
    if (0 > iRet) {
        return RET_FAILED_REGWR;
    }

    /* 正しく書き込まれたか確認する */
    uiReadData = i2c_smbus_read_word_data(m_fd, ucAddr);
    if (uiReadData != uiWriteData) {
        return RET_FAILED_MATCH;
    }

    return RET_SUCCESS;
}

int CSensorColor::siReadRegWord(
    unsigned char ucAddr,
    unsigned short* puiReadData)
{
    if (0 > m_fd) {
        return RET_INVALID_FD;
    }
    *puiReadData = i2c_smbus_read_word_data(m_fd, ucAddr);
    return RET_SUCCESS;
}

int CSensorColor::getRawValue(
    int* psiValRed,
    int* psiValGreen,
    int* psiValBlue,
    int* psiValWhite)
{
    /* 局所変数宣言 */
    int iRet;  // 戻り値
    unsigned short uiData;

    /* 赤成分の読出し */
    iRet = siReadRegWord(0x08, &uiData);
    if (RET_SUCCESS != iRet) {
        return iRet;
    }
    *psiValRed = uiData;

    /* 緑成分の読出し */
    iRet = siReadRegWord(0x09, &uiData);
    if (RET_SUCCESS != iRet) {
        return iRet;
    }
    *psiValGreen = uiData;

    /* 青成分の読出し */
    iRet = siReadRegWord(0x0A, &uiData);
    if (RET_SUCCESS != iRet) {
        return iRet;
    }
    *psiValBlue = uiData;

    /* 白成分の読出し */
    iRet = siReadRegWord(0x0B, &uiData);
    if (RET_SUCCESS != iRet) {
        return iRet;
    }
    *psiValWhite = uiData;

    return RET_SUCCESS;
}

enColor_t CSensorColor::getColor(void)
{
    int iRet;       // 戻り値

    int iRawRed;    // RAW赤成分
    int iRawGreen;  // RAW緑成分
    int iRawBlue;   // RAW青成分
    int iRawWhite;  // RAW明るさ（読み捨て）

#ifdef _DEBUG
    printf("[ENTER] >> getColor\n");
#endif /* _DEBUG */

    /* RAW値の読出し */
    iRet = getRawValue(&iRawRed, &iRawGreen, &iRawBlue, &iRawWhite);

    if (RET_SUCCESS != iRet) {
        return COL_BLK;
    }

    if((iRawRed > iRawGreen) && (iRawRed > iRawBlue)){
      return COL_RED;
    }
    else if((iRawGreen > iRawRed) && (iRawGreen > iRawBlue)){
      return COL_GRN;
    }
    else if((iRawBlue > iRawRed) && (iRawBlue > iRawGreen)){
      return COL_BLE;
    }

    return COL_WHT;
}
