/* --------------------------------------------------------------- */
/* ファイルインクルード */
/* --------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>  // exit()
#include <string.h>  // memset()
#include <unistd.h>  // usleep()
#include <signal.h>
#include <pthread.h>


#include <wiringPi.h>
//#include <softPwm.h>

#include "CommonDefine.h"
#include "PortDefine.h"
#include "CSwitchPhy.h"

//#define USEC_STABILIZE  (5)

/* --------------------------------------------------------------- */
/* 静的メンバ関数変数宣言 */
/* --------------------------------------------------------------- */
volatile bool CSwitchPhy::m_bContObserve;
volatile bool CSwitchPhy::m_bContWatch;
pthread_mutex_t mutex;

/* --------------------------------------------------------------- */
/* メンバ関数定義 */
/* --------------------------------------------------------------- */

/* *************************************************************** */
/* 関数名: CSwitchPhy */
/* 概要: コンストラクタ */
/* 引数: なし */
/* 戻り値: なし */
/* 説明: 初期設定を行なう */
/* *************************************************************** */
CSwitchPhy::CSwitchPhy()
{
    /* 局所変数宣言 */
    struct sigaction stSigAction;     // シグナル用


    /* シグナルハンドラの登録 */
    memset(&stSigAction, 0, sizeof(stSigAction));
    stSigAction.sa_handler = voSignalHandler;
    sigaction(SIGINT, &stSigAction, NULL);

#if 0
    /* 緊急スイッチの監視を開始する */
    m_bContWatch = true;
    (void)pthread_create(
        &m_thWatchEmer,
        NULL,
        pvWatchEmerThread,
        NULL
    );
#endif

    pthread_mutex_init(&mutex, NULL);

    /* スイッチ状態の初期化 */
    m_uiSwitchState = 0x0;

    return;
}



/* *************************************************************** */
/* 関数名: ~CSwitchPhy */
/* 概要: デストラクタ */
/* 引数: なし */
/* 戻り値: なし */
/* 説明: 後始末を行なう */
/* *************************************************************** */
CSwitchPhy::~CSwitchPhy()
{
    /* 観測スレッドを終了する */
    m_bContObserve = false;
    pthread_join(m_thObserve, NULL);

    /* 監視スレッドを終了する */
    m_bContWatch = false;
    pthread_join(m_thWatchEmer, NULL);

    return;
}


/* *************************************************************** */
/* 関数名: voSignalHandler */
/* 概要: シグナルハンドラ */
/* 引数: [in] iSigNo: シグナル要因 */
/* 戻り値: なし */
/* 説明: static, シグナルをハンドリングする */
/* *************************************************************** */
void CSwitchPhy::voSignalHandler(int iSigNo)
{
    printf("Handler\n");
    //m_bContObserve = false;
    //sleep(1);
    pthread_mutex_lock(&mutex);

    // PWM設定を停止
    pwmSetClock(100);
    pwmSetRange(1024);
    pwmWrite(P_MO_STEP, 0);

    // モータ停止
    digitalWrite(P_MO_SLEEP, 0);

    exit(RET_EMER_STOP);
    pthread_mutex_unlock(&mutex);
}

/* *************************************************************** */
/* 関数名: initDevice */
/* 概要: GPIO初期化 */
/* 引数: [in] bInitGpio: wiringPiSetupを行なうか否か */
/* 戻り値: なし */
/* 説明: GPIOの設定を行なう */
/* *************************************************************** */
int CSwitchPhy::initDevice(bool bInitGpio)
{
    /* 局所変数宣言 */
    int iRet;

    /* GPIOの初期化 */
    if (true == bInitGpio) {
        iRet = wiringPiSetup();

        if (-1 == iRet) {
            return RET_FAILED;
        }
    }


    /* GPIOの入出力設定 */
    pinMode(P_UT_BTN1, INPUT);
    pinMode(P_UT_BTN2, INPUT);
    pinMode(P_UT_USR1, INPUT);
    pinMode(P_UT_USR2, INPUT);
    pinMode(P_UT_EMR1, INPUT);
    pinMode(P_UT_EMR2, INPUT);


        /* 緊急スイッチの監視を開始する */
    m_bContWatch = true;
    (void)pthread_create(
        &m_thWatchEmer,
        NULL,
        pvWatchEmerThread,
        NULL
    );

    return RET_SUCCESS;
}


/* *************************************************************** */
/* 関数名: getInstance */
/* 概要: インスタンスの取得 */
/* 引数: なし */
/* 戻り値: インスタンスの参照 */
/* 説明: 唯一のインスタンスの取得を行なう */
/* *************************************************************** */
CSwitchPhy& CSwitchPhy::getInstance(void)
{
    static CSwitchPhy inst;
    return inst;
}

/* *************************************************************** */
/* 関数名: getSwitchState */
/* 概要: スイッチ状態の取得 */
/* 引数: [in] pvParam: 観測間隔（要キャスト） */
/* 戻り値: なし */
/* 説明: static, GPIOに接続されている物理スイッチの状態を観測する */
/* *************************************************************** */
unsigned int CSwitchPhy::getSwitchState(void)
{
#ifdef _DEBUG
    if (SW_STATE_ON_USR1 == (m_uiSwitchState & SW_STATE_ON_USR1)) {
        printf("SW_STATE_ON_USR1\n");
    }
    if (SW_STATE_ON_USR2 == (m_uiSwitchState & SW_STATE_ON_USR2)) {
        printf("SW_STATE_ON_USR2\n");
    }
    if (SW_STATE_ON_BTN1 == (m_uiSwitchState & SW_STATE_ON_BTN1)) {
        printf("SW_STATE_ON_BTN1\n");
    }
    if (SW_STATE_ON_BTN2 == (m_uiSwitchState & SW_STATE_ON_BTN2)) {
        printf("SW_STATE_ON_BTN2\n");
    }
#endif /* _DEBUG */

    return m_uiSwitchState;
}


/* *************************************************************** */
/* 関数名: startObserve */
/* 概要: スイッチ状態の観測開始 */
/* 引数: [in] uiInterv: 観測間隔 */
/* 戻り値: なし */
/* 説明: 観測用のスレッドを生成して観測を開始する */
/* *************************************************************** */
int CSwitchPhy::startObserve(unsigned int uiInterv)
{
    /* 局所変数宣言 */
    int iRet;

    /* インターバルの設定 */
    m_uiIntvObserve = uiInterv;

    /* スレッドの生成 */
    m_bContObserve = true;
    iRet = pthread_create(
        &m_thObserve,
        NULL,
        pvObserveThread,
        this
    );

    return (0 > iRet) ? RET_FAILED : RET_SUCCESS;

}

/* *************************************************************** */
/* 関数名: pvObserveThread */
/* 概要: 緊急スイッチを監視するスレッド */
/* 引数: [in] pvParam: 未使用 */
/* 戻り値: なし */
/* 説明: static, 緊急スイッチの状態を観測する */
/* *************************************************************** */
void* CSwitchPhy::pvWatchEmerThread(void* pvParam)
{
    while (true == m_bContWatch) {
        usleep(MSEC_INTV_OBSERVE * 1000);

        /* *** 緊急停止スイッチ *** */
        /* プルアップしているので0のとき感知 */
        if ((0 == digitalRead(P_UT_EMR1)) ||
            (0 == digitalRead(P_UT_EMR2))) {
          //if (0 == digitalRead(P_UT_EMR1)) {

          printf("HIT\n");

          /* SIGINTを発生させる */
          raise(SIGINT);
        }
    }
    return 0;
}

/* *************************************************************** */
/* 関数名: pvObserveThread */
/* 概要: スイッチの状態を観測するスレッド */
/* 引数: [in] pvParam: 観測間隔（要キャスト） */
/* 戻り値: なし */
/* 説明: static, GPIOに接続されている物理スイッチの状態を観測する */
/* *************************************************************** */
void* CSwitchPhy::pvObserveThread(void* pvParam)
{
    /* 局所変数宣言 */
    //int iVal;
    CSwitchPhy* pInst = (CSwitchPhy*)pvParam;
    unsigned int* puiState = &(pInst->m_uiSwitchState);
    const unsigned int cuiInterv = pInst->m_uiIntvObserve;    // 観測間隔



    while (true == m_bContObserve) {

        //printf(".\n");
        usleep(cuiInterv * 1000);



        /* *** ユーザスイッチ *** */
        /* プルダウンしているので1のとき感知 */
        *puiState = (1 == digitalRead(P_UT_USR1))
                  ? *puiState | SW_STATE_ON_USR1
                  : *puiState & ~SW_STATE_ON_USR1;
        *puiState = (1 == digitalRead(P_UT_USR2))
                  ? *puiState | SW_STATE_ON_USR2
                  : *puiState & ~SW_STATE_ON_USR2;

        /* *** タクトスイッチ *** */
        /* プルアップしているので0のとき感知 */
        *puiState = (0 == digitalRead(P_UT_BTN1))
                  ? *puiState | SW_STATE_ON_BTN1
                  : *puiState & ~SW_STATE_ON_BTN1;
        *puiState = (0 == digitalRead(P_UT_BTN2))
                  ? *puiState | SW_STATE_ON_BTN2
                  : *puiState & ~SW_STATE_ON_BTN2;
#if 0
        if (1 == digitalRead(P_UT_USR1)) {
            m_uiSwitchState |= SW_STATE_ON_USR1;
        }
        else {
            m_uiSwitchState &= ~SW_STATE_ON_USR1;
        }
        if (1 == digitalRead(P_UT_USR2)) {
            m_uiSwitchState |= SW_STATE_ON_USR2;
        }
        else {
            m_uiSwitchState &= ~SW_STATE_ON_USR2;
        }

        /* *** タクトスイッチ *** */
        /* プルダウンしているので1のとき感知 */
        if (1 == digitalRead(P_UT_BTN1)) {
            m_uiSwitchState |= SW_STATE_ON_BTN1;
        }
        else {
            m_uiSwitchState &= ~SW_STATE_ON_BTN1;
        }
        if (1 == digitalRead(P_UT_BTN2)) {
            m_uiSwitchState |= SW_STATE_ON_BTN2;
        }
        else {
            m_uiSwitchState &= ~SW_STATE_ON_BTN2;
        }
#endif /* 0 */
    }
    return 0;
}
