#ifndef __C_SWITCH_PHY_H__

/* *** 2重イングルード禁止 *** */
#define __C_SWITCH_PHY_H__

/* *** ファイルインクルード *** */
#include <pthread.h>
//#include "PortDefine.h"


/* *** 定数定義 *** */
#define SW_STATE_ON_USR1  (0x01)
#define SW_STATE_ON_USR2  (0x02)
#define SW_STATE_ON_EMR1  (0x04)
#define SW_STATE_ON_EMR2  (0x08)
#define SW_STATE_ON_BTN1  (0x10)
#define SW_STATE_ON_BTN2  (0x20)

/* *** 共通定義 *** */
#define MSEC_INTV_OBSERVE  (50)
#define RET_EMER_STOP      (-21)


/* *** 前方宣言 *** */
//enum _enColor_t;

extern pthread_mutex_t mutex;

/* *** クラス定義 *** */
class CSwitchPhy {
    /* Variables */
    public:
    private:
        static volatile bool m_bContObserve;    // 観測継続フラグ
        static volatile bool m_bContWatch;      // 監視継続フラグ

        pthread_t  m_thObserve;    // 観測用スレッド識別子
        pthread_t  m_thWatchEmer;  // 緊急スイッチ監視スレッド識別子

        unsigned int m_uiSwitchState;  // スイッチの状態
        unsigned int m_uiIntvObserve;  // 観測の間隔 [ms]

    /* Functions */
    public:
        /* インスタンスの取得 */
        static CSwitchPhy& getInstance(void);
        /* デバイスの初期化 */
        int initDevice(bool bInitGpio = false);
        /* スイッチ状態の観測開始 */
        int startObserve(unsigned int uiInterv = MSEC_INTV_OBSERVE);

        unsigned int getSwitchState(void);

    private:
        CSwitchPhy();     // コンストラクタ
        ~CSwitchPhy();    // デストラクタ
        CSwitchPhy(const CSwitchPhy&);             // コピーコンストラクタ（定義しない）
        CSwitchPhy& operator=(const CSwitchPhy&);  // コピー代入演算子（定義しない）

        static void voSignalHandler(int iSigNo);
        static void* pvObserveThread(void* pvParam);
        static void* pvWatchEmerThread(void* pvParam);


};


#endif /* __C_SWITCH_PHY_H__ */
