/**
 * @file Controller.cpp
 * @brief 制御
 * @author Naoki Okayama
 * @date 2017/06/01
 */
#define __DEBUG1__
//#define __DEBUG2__

#include <iostream>
#include "MotorController.h"
#include "CLedColor.h"
#include "CSensorColor.h"
//#include "CSensorLaser.h"
#include "CNetMqtt.h"
#include "Controller.h"
#include "CSwitchPhy.h"

#define SENSOR_COLOR_ADDR ((unsigned char)0x10)
//#define SENSOR_LASER_ADDR ((unsigned char)0x29)

//////////////////////////////////////////////////////////
// public
//////////////////////////////////////////////////////////
/**
 * @brief コンストラクタ
 * @param -
 * @return -
 * @sa -
 */
Controller::Controller(void):
    netMqtt(CNetMqtt::getInstance()),
    switchPhy(CSwitchPhy::getInstance()){
    this->motorController = new MotorController();
    this->ledColor = new CLedColor();
    this->sensorColor = new CSensorColor(SENSOR_COLOR_ADDR);
    //    this->sensorLaser = new CSensorLaser(SENSOR_LASER_ADDR);

    this->point[0] = 0;
    this->point[1] = 0;
    this->point[2] = 0;
    this->point[3] = 0;
}

/**
 * @brief デストラクタ
 * @param -
 * @return -
 * @sa -
 */
Controller::~Controller(void){
    int iRet;
#ifndef __DEBUG1__
    //BYE
    iRet = netMqtt.publishMessage(PLAYER_BYE);
    if(iRet != RET_SUCCESS){
        std::cout << "[ERR]publish Bye" << std::endl;
    }
#endif
    //delete sensorLaser;
    delete sensorColor;
    delete ledColor;
    delete motorController;
}

/**
 * @brief コントローラの初期化
 * @param -
 * @return 状態（成功：RET_SUCCESS 失敗：RET_FAILED）
 * @sa -
 */
int Controller::initController(void){
    int iRet;
    enMsgId_t enMsgId;

    this->xRef = 0.0;

    if( (this->motorController != NULL)
            && (this->ledColor != NULL)
            && (this->sensorColor != NULL)
	//&& (this->sensorLaser != NULL)
        ){
        if(this->motorController->initMotorController(true) != RET_SUCCESS){
            std::cout << "[ERR] init MotorController" << std::endl;
            return RET_FAILED;
        }

        if(this->ledColor->initDevice(false) != RET_SUCCESS){
            std::cout << "[ERR] init LedColor" << std::endl;
            return RET_FAILED;
        }

        if(this->sensorColor->initDevice() != RET_SUCCESS){
            std::cout << "[ERR] init SensorColor" << std::endl;
            return RET_FAILED;
        }
#if 0
        if(this->sensorLaser->initDevice() != RET_SUCCESS){
            std::cout << "[ERR] init SensorLaser" << std::endl;
            return RET_FAILED;
        }

        if(this->sensorLaser->prepDevice() != RET_SUCCESS){
            std::cout << "[ERR] init SensorLaser" << std::endl;
            return RET_FAILED;
        }
#endif
#ifndef __DEBUG2__
        if(this->switchPhy.initDevice() != RET_SUCCESS){
          std::cout << "[ERR] init switchPhy" << std::endl;
          return RET_FAILED;
        }

        if(this->switchPhy.startObserve() != RET_SUCCESS){
          std::cout << "[ERR] start Observe" << std::endl;
          return RET_FAILED;
        }
#endif
    }else{
        std::cout << "[ERR] Object's is NULL" << std::endl;
        return RET_FAILED;
    }

    this->lightColor(COL_BLK);
#ifndef __DEBUG1__
    // MQTT 接続
#if (JUDGE == A)
    netMqtt.initConnect("PLAYER", "192.168.20.21");
#elif (JUDGE == B)
    netMqtt.initConnect("PLAYER", "192.168.20.22");
#elif (JUDGE == C)
    netMqtt.initConnect("PLAYER", "192.168.20.23");
#elif (JUDGE == D)
    netMqtt.initConnect("PLAYER", "192.168.20.24");
#elif (JUDGE == E)
    netMqtt.initConnect("PLAYER", "192.168.20.25");
#else
    netMqtt.initConnect("PLAYER", "192.168.20.21");
#endif

#if (TEAM == A)
    netMqtt.setTopic("A");
    netMqtt.subscrTopic("A");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"A");
#elif (TEAM == B)
    netMqtt.setTopic("B");
    netMqtt.subscrTopic("B");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"B");
#elif (TEAM == C)
    netMqtt.setTopic("C");
    netMqtt.subscrTopic("C");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"C");
#elif (TEAM == D)
    netMqtt.setTopic("D");
    netMqtt.subscrTopic("D");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"D");

#elif (TEAM == E)
    netMqtt.setTopic("E");
    netMqtt.subscrTopic("E");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"E");

#elif (TEAM == G)
    netMqtt.setTopic("G");
    netMqtt.subscrTopic("G");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"G");

#elif (TEAM == H)
    netMqtt.setTopic("H");
    netMqtt.subscrTopic("H");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"H");

#elif (TEAM == I)
    netMqtt.setTopic("I");
    netMqtt.subscrTopic("I");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"I");

#elif (TEAM == J)
    netMqtt.setTopic("J");
    netMqtt.subscrTopic("J");
    // HELLO
    iRet = netMqtt.publishMessage(PLAYER_HELLO,"J");

#endif


    if(iRet != RET_SUCCESS){
        std::cout << "[ERR]publish Hello" << std::endl;
        return RET_FAILED;
    }


    // 応答待ち TODO:タイムアウト
    do{
        netMqtt.dequeueMessage(&enMsgId);
    }while(enMsgId != JUDGE_HELLO);
#endif

    return RET_SUCCESS;
}

/**
 * @brief 目標位置設定
 * @param[in] xRef 目標位置[cm]
 * @return -
 * @sa -
 */
void Controller::setPosition(float xRef){
    this->xRef = xRef;
}

/**
 * @brief カラーセンサの生値の設定
 * @param[out] psiValRed 赤色の生値
 * @param[out] psiValGreen 緑色の生値
 * @param[out] psiValBlue 青色の生値
 * @param[out] psiValWhite 白色の生値
 * @return -
 * @sa -
 */
void Controller::getRawValue(int* psiValRed, int* psiValGreen, int* psiValBlue, int* psiValWhite){
    *psiValRed = this->psiValRed;
    *psiValGreen = this->psiValGreen;
    *psiValBlue = this->psiValBlue;
    *psiValWhite = this->psiValWhite;
}

/**
 * @brief カラーセンサの値の取得
 * @param -
 * @return 取得した色（赤：COL_RED 青：COL_BLU 緑：COL_GRN 黄：COL_YLW 黒：COL_BLK）
 * @sa -
 */
_enColor_t Controller::getColor(void){
    return this->color;
}

#if 0
/**
 * @brief レーザーセンサの値の取得
 * @param -
 * @return 距離[cm]
 * @sa -
 */
int Controller::getDistance(void){
    return this->distance;
}
#endif

/**
 * @brief LEDSliderの位置の取得
 * @param -
 * @return 位置[cm]
 * @sa -
 */
float Controller::getPosition(void){
    return this->position;
}

/**
 * @brief カラーLEDの点灯
 * @param[in] color 点灯する色（赤：COL_RED 青：COL_BLU 緑：COL_GRN 黄：COL_YLW 黒：COL_BLK）
 * @return 点灯状態（成功：RET_SUCCESS 失敗：RET_FAILED）
 * @sa -
 */
int Controller::lightColor(_enColor_t color){
    return ledColor->lightColor(color);
}

/**
 * @brief Mqttのメッセージキューからデキューする
 * @param -
 * @return -
 * @sa -
 */
void Controller::dequeueMessage(void){
    netMqtt.dequeueMessage(&enMsgId);
}

/**
 * @brief メッセージを取得する
 * @param -
 * @return メッセージ
 * @sa -
 */
enMsgId_t Controller::getMessage(void){
    return this->enMsgId;
}



/**
 * @brief 実行関数
 * @param -
 * @return -
 * @sa -
 */
void Controller::run(void){
    this->motorController->movePosition(this->xRef);

#ifndef __DEBUG1__
    this->dequeueMessage();
#endif

    this->position = this->motorController->getPosition();
    this->color = this->sensorColor->getColor();
    this->sensorColor->getRawValue(&(this->psiValRed), &(this->psiValGreen), &(this->psiValBlue), &(this->psiValWhite));
    //this->distance = this->sensorLaser->getDistance();
    this->switchState = this->switchPhy.getSwitchState();
}

/**
 * @brief 得点表更新の要求
 * @param -
 * @return -
 * @sa -
 */
void Controller::requestPoint(void){
  this->netMqtt.publishMessage(PLAYER_UPDATE);
}

/**
 * @brief 得点表更新
 * @param -
 * @return -
 * @sa -
 */
void Controller::updatePoint(void){
  char content[256];

  this->netMqtt.getContent(content, 256);
  sscanf(content, "%d,%d,%d,%d",&this->point[0],&this->point[1],&this->point[2],&this->point[3]);
}



/**
 * @brief チェック開始
 * @param[in] sensePosition センシングした場所を指定する。POS_A、POS_B、POS_C、POS_Dで指定する。
 * @return -
 * @sa -
 */
void Controller::startCheck(enLEDSliderPos_t sensePosition){
  switch(sensePosition){
  case POS_A:
    this->netMqtt.publishMessage(PLAYER_START, "A");
    break;
  case POS_B:
    this->netMqtt.publishMessage(PLAYER_START, "B");
    break;
  case POS_C:
    this->netMqtt.publishMessage(PLAYER_START, "C");
    break;
  case POS_D:
    this->netMqtt.publishMessage(PLAYER_START, "D");
    break;
  default:
    break;
  }
}

/**
 * @brief スイッチの状態取得
 * @param -
 * @return スイッチの状態
 * @sa -
 */
unsigned int Controller::getSwitchState(void){
  return this->switchState;
}
