/**
 * @file Controller.h
 * @brief 制御
 * @author Naoki Okayama
 * @date 2017/06/01
 */

#ifndef __CONTROLLER_H__
#define __CONTROLLER_H__

#include "CommonDefine.h"

#define A 0
#define B 1
#define C 2
#define D 3
#define E 4
#define F 5
#define G 6
#define H 7
#define I 8
#define J 9

class MotorController;
class CLedColor;
class CSensorColor;
//class CSensorLaser;
class IEvent;
class CNetMqtt;
class CSwitchPhy;

class Controller{
public:
    int cnt;
    int point[4];

    MotorController *motorController;                        /*!< MotorControllerオブジェクト */
    CLedColor *ledColor;                                     /*!< CLedColorオブジェクト */
    CSensorColor *sensorColor;                               /*!< CSensorColorオブジェクト */
    //CSensorLaser *sensorLaser;                               /*!< CSensorLaserオブジェクト */
    CNetMqtt &netMqtt;
    CSwitchPhy &switchPhy;

    float position;
    int distance;
    enColor_t color;
    int psiValRed;
    int psiValGreen;
    int psiValBlue;
    int psiValWhite;
    unsigned int switchState;

    Controller(void);
    ~Controller(void);

    int initController(void);
    void setPosition(float xRef);
    void getRawValue(int* psiValRed, int* psiValGreen, int* psiValBlue, int* psiValWhite);
    enColor_t getColor(void);
    //int getDistance(void);
    float getPosition(void);
    int lightColor(_enColor_t enColor);
    void run(void);
    void updatePoint(void);
    void requestPoint(void);
    void startCheck(enLEDSliderPos_t sensePosition);
    enMsgId_t getMessage(void);
    unsigned int getSwitchState(void);

private:
    float xRef;
    enum _enMsgId_t enMsgId;

    void movePosition(void);
    void dequeueMessage(void);
};

#endif
